#!/usr/bin python3
# Copyright (c) 2023 RheoData and/or its affiliates.  
# All rights reserved.
# and is released under the "MIT License Agreement". Please see the LICENSE
# file that should have been included as part of this package.
#
# Since:        Aug 2023
# Author:       Bobby Curtis <bobby.curtis@rheodata.com>
# Description:  GoldenGate Services (GGS) Monitor
#
# DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
#
import requests
import json 
import os
from dotenv import load_dotenv
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime
import time

#Load up the .env varable file.
load_dotenv()

#Use the variables defined in .env file and build cURL header.
vURL = os.getenv('URL')
vAuth = os.getenv('AUTH')
vVersion = os.getenv('VERSION')
vRefresh = os.getenv('REFRESH')

headers = {
    'Authorization': vAuth,
    'Content-Type': 'text/plain'
}

#Disable warnings, security related
requests.packages.urllib3.disable_warnings()

#Function to clear screen
def cls():
    os.system('clear')

#Function to define header for screen output
def header():
        vDatetime = datetime.now()
        vTitle = "GoldenGate Service Monitor"
        vVer = "Version: " + vVersion
        vCo = "RheoData"
        vCopyright = "Copyright 2023"
        vRefreshTime = "Date:" + vDatetime.isoformat()
        print(vTitle.center(135))
        print(vVer.center(135))
        print(vCo.center(135))
        print(vCopyright.center(135))
        print(vRefreshTime.center(135))
        print("*"*156)
        print("")
        print("{:<8} {:<8} {:<8} {:<8} {:<8} {:<8} {:<1} {:<15} {:<10} {:<10} {:<10} {:<25} {:<25}".format('Extract', 'Status', 'Lag', 'Replicat', 'Status', 'Lag', '|', 'EPostion', 'RTrail', 'RSeq', 'ROffset', 'ELastStarted', 'RLastStarted'))
        print("{:<8} {:<8} {:<8} {:<8} {:<8} {:<8} {:<1} {:<15} {:<10} {:<10} {:<10} {:<25} {:<25}".format('='*8, '='*7, '='*8, '='*8, '='*7, '='*8, '|', '='*15, '='*7, '='*6, '='*10, '='*25, '='*25))

#Function to get and process JSON responses
def get_json_response(url):
    response = requests.get(url, headers=headers, verify=False)
    return json.loads(response.text)

#Function to get processes and process the JSON
def get_process(url, key):
    response = requests.get(url, headers=headers, verify=False)
    jsonResponse = json.loads(response.text)
    try:
        names = [jsonResponse["response"]["items"][i][key] for i in range(len(jsonResponse["response"]["items"]))]
    except KeyError:
        names = []
    return names

#Main function that proceses the extract and replicats and does the processing in parallel
#after which will sleep for 15 seconds and refresh the screen as needed.        
def main():
    while True:
        cls()
        header()

        vExtracts = get_process(f"{vURL}/services/v2/extracts", "name")
        vReplicats = get_process(f"{vURL}/services/v2/replicats", "name")
        
        with ThreadPoolExecutor(max_workers=5) as executor:
            results = []
            for (e, r) in zip(vExtracts, vReplicats):
                results.append(executor.submit(process_extract_replicat, e, r))

            for result in results:
                print(result.result())

        time.sleep(int(vRefresh))

#Function that defines the URLs to process and gather required information.
def process_extract_replicat(e, r):
    urls = [
        (f"{vURL}/services/v2/replicats/{r}/info/status", "status"),
        (f"{vURL}/services/v2/extracts/{e}/info/status", "status"),
    ]
    responses = [get_json_response(url) for url, key in urls]
    #print(responses)
    try:
        #Replicat Status Information
        rStatus = responses[0]["response"]["status"]
        rStartTime = responses[0]["response"]["lastStarted"]
        rTrail = responses[0]["response"]["position"]["name"]
        rSequence = responses[0]["response"]["position"]["sequence"]
        rOffset = responses[0]["response"]["position"]["offset"]
        rLag = responses[0]["response"]["lag"]
        #Extract Status Information
        eStatus = responses[1]["response"]["status"]
        eStartTime = responses[1]["response"]["lastStarted"]
        ePostion = responses[1]["response"]["position"]
        eLag = responses[1]["response"]["lag"]
    except (KeyError, ValueError):
        return "{:<8} {:<8} {:<8} {:<8} {:<8} {:<8} {:<1} {:<15} {:<10} {:<10} {:<10} {:<25} {:<25}".format(e, "NULL", "NULL", r, "NULL", "NULL", "|", "NULL", "NULL", "NULL", "NULL", "NULL", "NULL")
    except KeyboardInterrupt:
        print("Interrupted")
    
    try:
        return "{:<8} {:<8} {:<8} {:<8} {:<8} {:<8} {:<1} {:<15} {:<10} {:<10} {:<10} {:<25} {:<25}".format(e, eStatus.upper(), eLag, r, rStatus.upper(), rLag, '|', ePostion, rTrail, rSequence, rOffset, eStartTime, rStartTime )
    except Exception as e:
        print("An error occured:", e)

#Execute the main funtion.
if __name__ == "__main__":
    main()